@@ -15,7 +15,9 @@
  * You should have received a copy of the GNU Lesser General Public
  * License along with libnautilus-extension.  If not, see <https://www.gnu.org/licenses/>.
  */
-#ifndef NAUTILUS_EXTENSION_H
+
+#pragma once
+
 #define NAUTILUS_EXTENSION_H
 
 #include <libnautilus-extension/nautilus-column-provider.h>
@@ -34,6 +36,8 @@
  * @title: Extension entry points
  */
 
+G_BEGIN_DECLS
+
 /**
  * nautilus_module_initialize: (skip)
  * @module: a #GTypeModule used in type registration
@@ -59,4 +63,4 @@ void nautilus_module_shutdown   (void);
 void nautilus_module_list_types (const GType **types,
                                  int          *num_types);
 
-#endif
+G_END_DECLS
