@@ -22,6 +22,7 @@
 #if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #endif
 
+#include "nautilusgtkplacessidebarprivate.h"
 
 G_BEGIN_DECLS
 
@@ -34,45 +35,16 @@ G_BEGIN_DECLS
 
 typedef struct _NautilusGtkPlacesView NautilusGtkPlacesView;
 typedef struct _NautilusGtkPlacesViewClass NautilusGtkPlacesViewClass;
-typedef struct _NautilusGtkPlacesViewPrivate NautilusGtkPlacesViewPrivate;
-
-struct _NautilusGtkPlacesViewClass
-{
-  GtkBoxClass parent_class;
-
-  void     (* open_location)        (NautilusGtkPlacesView          *view,
-                                     GFile                  *location,
-                                     GtkPlacesOpenFlags  open_flags);
-
-  void    (* show_error_message)     (GtkPlacesSidebar      *sidebar,
-                                      const gchar           *primary,
-                                      const gchar           *secondary);
-
-  /*< private >*/
-
-  /* Padding for future expansion */
-  gpointer reserved[10];
-};
-
-struct _NautilusGtkPlacesView
-{
-  GtkBox parent_instance;
-};
 
 GType              nautilus_gtk_places_view_get_type                      (void) G_GNUC_CONST;
 
-GtkPlacesOpenFlags nautilus_gtk_places_view_get_open_flags                (NautilusGtkPlacesView      *view);
+NautilusGtkPlacesOpenFlags nautilus_gtk_places_view_get_open_flags                (NautilusGtkPlacesView      *view);
 void               nautilus_gtk_places_view_set_open_flags                (NautilusGtkPlacesView      *view,
-                                                                  GtkPlacesOpenFlags  flags);
+                                                                  NautilusGtkPlacesOpenFlags  flags);
 
-const gchar*       nautilus_gtk_places_view_get_search_query              (NautilusGtkPlacesView      *view);
+const char *       nautilus_gtk_places_view_get_search_query              (NautilusGtkPlacesView      *view);
 void               nautilus_gtk_places_view_set_search_query              (NautilusGtkPlacesView      *view,
-                                                                  const gchar        *query_text);
-
-gboolean           nautilus_gtk_places_view_get_local_only                (NautilusGtkPlacesView         *view);
-
-void               nautilus_gtk_places_view_set_local_only                (NautilusGtkPlacesView         *view,
-                                                                  gboolean               local_only);
+                                                                  const char         *query_text);
 
 gboolean           nautilus_gtk_places_view_get_loading                   (NautilusGtkPlacesView         *view);
 
