@@ -219,7 +219,7 @@ typedef struct {
 
 #define NAUTILUS_TYPE_BATCH_RENAME_DIALOG (nautilus_batch_rename_dialog_get_type())
 
-G_DECLARE_FINAL_TYPE (NautilusBatchRenameDialog, nautilus_batch_rename_dialog, NAUTILUS, BATCH_RENAME_DIALOG, GtkDialog);
+G_DECLARE_FINAL_TYPE (NautilusBatchRenameDialog, nautilus_batch_rename_dialog, NAUTILUS, BATCH_RENAME_DIALOG, AdwWindow);
 
 GtkWidget*      nautilus_batch_rename_dialog_new                      (GList                     *selection,
                                                                        NautilusDirectory         *directory,
@@ -229,4 +229,4 @@ void            nautilus_batch_rename_dialog_query_finished           (NautilusB
                                                                        GHashTable                *hash_table,
                                                                        GList                     *selection_metadata);
 
-G_END_DECLS
\ No newline at end of file
+G_END_DECLS
