@@ -57,7 +57,7 @@ gboolean file_name_conflicts_with_results       (GList        *selection,
                                                  GString      *old_name,
                                                  gchar        *parent_uri);
 
-GString* batch_rename_replace_label_text        (gchar             *label,
+GString* batch_rename_replace_label_text        (const char        *label,
                                                  const gchar       *substr);
 
 gchar*   batch_rename_get_tag_text_representation (TagConstants tag_constants);
@@ -67,4 +67,4 @@ void batch_rename_sort_lists_for_rename (GList    **selection,
                                          GList    **old_names,
                                          GList    **new_files,
                                          GList    **old_files,
-                                         gboolean   is_undo_redo);
\ No newline at end of file
+                                         gboolean   is_undo_redo);
