@@ -22,26 +22,15 @@ static gboolean
 check_same_fs (NautilusFile *file1,
                NautilusFile *file2)
 {
-    char *id1, *id2;
-    gboolean result;
-
-    result = FALSE;
-
-    if (file1 != NULL && file2 != NULL)
+    if (file1 == NULL || file2 == NULL)
     {
-        id1 = nautilus_file_get_filesystem_id (file1);
-        id2 = nautilus_file_get_filesystem_id (file2);
-
-        if (id1 != NULL && id2 != NULL)
-        {
-            result = (strcmp (id1, id2) == 0);
-        }
-
-        g_free (id1);
-        g_free (id2);
+        return FALSE;
     }
 
-    return result;
+    const char *id1 = nautilus_file_get_filesystem_id (file1);
+    const char *id2 = nautilus_file_get_filesystem_id (file2);
+
+    return id1 != NULL && id2 != NULL && strcmp (id1, id2) == 0;
 }
 
 static gboolean
@@ -147,8 +136,8 @@ nautilus_drag_drop_action_ask (GtkWidget     *widget,
 
     /* We don't have pointer coords here. Just pick the center of the widget. */
     gtk_popover_set_pointing_to (GTK_POPOVER (popover),
-                                 &(GdkRectangle){ .x = 0.5 * gtk_widget_get_allocated_width (widget),
-                                                  .y = 0.5 * gtk_widget_get_allocated_height (widget),
+                                 &(GdkRectangle){ .x = 0.5 * gtk_widget_get_width (widget),
+                                                  .y = 0.5 * gtk_widget_get_height (widget),
                                                   .width = 0, .height = 0 });
 
     gtk_popover_popup (GTK_POPOVER (popover));
@@ -265,7 +254,7 @@ nautilus_dnd_perform_drop (NautilusFilesView *view,
         g_autofree char *dest_uri = g_file_get_uri (target_location);
         GdkTexture *texture = g_value_get_object (value);
 
-        nautilus_file_view_save_image_from_texture (view, texture, dest_uri, _("Dropped image"));
+        nautilus_file_view_save_image_from_texture (view, texture, dest_uri, _("Dropped Image"));
     }
     else if (G_VALUE_HOLDS (value, GDK_TYPE_FILE_LIST))
     {
