@@ -57,8 +57,10 @@ struct NautilusFilePrivate
 	int sort_order;
 	
 	guint32 permissions;
-	int uid; /* -1 is none */
-	int gid; /* -1 is none */
+	gboolean has_uid;
+	uid_t uid;
+	gboolean has_gid;
+	gid_t gid;
 
 	GRefString *owner;
 	GRefString *owner_real;
@@ -73,7 +75,6 @@ struct NautilusFilePrivate
 	GRefString *mime_type;
 	
 	char *selinux_context;
-	char *description;
 	
 	GError *get_info_error;
 	
@@ -90,8 +91,6 @@ struct NautilusFilePrivate
 	GdkPixbuf *thumbnail;
 	time_t thumbnail_mtime;
 
-	GList *mime_list; /* If this is a directory, the list of MIME types in it. */
-
 	/* Info you might get from a link (.desktop, .directory or nautilus link) */
 	GIcon *custom_icon;
 	char *activation_uri;
@@ -130,6 +129,7 @@ struct NautilusFilePrivate
            many NautilusFile objects. */
 
 	guint unconfirmed                   : 1;
+	guint has_been_unmounted            : 1;
 	guint is_gone                       : 1;
 	/* Set when emitting files_added on the directory to make sure we
 	   add a file, and only once */
@@ -151,10 +151,6 @@ struct NautilusFilePrivate
            intermediate values for this attribute, we do actually
            forget it rather than invalidating. */
 
-	guint got_mime_list                 : 1;
-	guint mime_list_failed              : 1;
-	guint mime_list_is_up_to_date       : 1;
-
 	guint mount_is_up_to_date           : 1;
 	
 	guint got_custom_display_name       : 1;
@@ -191,7 +187,6 @@ struct NautilusFilePrivate
 	guint filesystem_use_preview        : 2; /* GFilesystemPreviewType */
 	guint filesystem_info_is_up_to_date : 1;
 	guint filesystem_remote             : 1;
-	GRefString     *filesystem_type;
 
 	time_t trash_time; /* 0 is unknown */
 	time_t recency; /* 0 is unknown */
@@ -206,8 +201,8 @@ struct NautilusFilePrivate
 typedef struct {
 	NautilusFile *file;
 	GList *files;
-	gint renamed_files;
-	gint skipped_files;
+	guint renamed_files;
+	guint skipped_files;
 	GCancellable *cancellable;
 	NautilusFileOperationCallback callback;
 	gpointer callback_data;
@@ -220,12 +215,15 @@ typedef struct {
 
 NautilusFile *nautilus_file_new_from_info                  (NautilusDirectory      *directory,
 							    GFileInfo              *info);
+NautilusFile *nautilus_file_new_from_filename              (NautilusDirectory *directory,
+                                                            const char        *filename,
+                                                            gboolean           self_owned);
 void          nautilus_file_emit_changed                   (NautilusFile           *file);
+void          nautilus_file_mark_unmounted                 (NautilusFile           *file);
 void          nautilus_file_mark_gone                      (NautilusFile           *file);
 
-gboolean      nautilus_file_get_date                       (NautilusFile           *file,
-							    NautilusDateType        date_type,
-							    time_t                 *date);
+GDateTime *   nautilus_file_get_date                       (NautilusFile           *file,
+							    NautilusDateType        date_type);
 void          nautilus_file_updated_deep_count_in_progress (NautilusFile           *file);
 
 
@@ -262,7 +260,7 @@ void                   nautilus_file_invalidate_attributes_internal     (Nautilu
 									 NautilusFileAttributes  file_attributes);
 NautilusFileAttributes nautilus_file_get_all_attributes                 (void);
 gboolean               nautilus_file_is_self_owned                      (NautilusFile           *file);
-void                   nautilus_file_invalidate_count_and_mime_list     (NautilusFile           *file);
+void                   nautilus_file_invalidate_count                   (NautilusFile           *file);
 gboolean               nautilus_file_rename_in_progress                 (NautilusFile           *file);
 void                   nautilus_file_invalidate_extension_info_internal (NautilusFile           *file);
 void                   nautilus_file_info_providers_done                (NautilusFile           *file);
