@@ -32,8 +32,6 @@ void nautilus_files_view_handle_text_drop         (NautilusFilesView *view,
                                                    const char        *text,
                                                    const char        *target_uri,
                                                    GdkDragAction      action);
-void nautilus_files_view_handle_hover             (NautilusFilesView *view,
-                                                   const char        *target_uri);
 
 void nautilus_files_view_drop_proxy_received_uris (NautilusFilesView *view,
                                                    const GList       *uris,
