@@ -16,7 +16,6 @@ struct _NautilusGridView
 
     GtkGridView *view_ui;
 
-    GActionGroup *action_group;
     gint zoom_level;
 
     gboolean directories_first;
@@ -29,11 +28,23 @@ struct _NautilusGridView
 
 G_DEFINE_TYPE (NautilusGridView, nautilus_grid_view, NAUTILUS_TYPE_LIST_BASE)
 
-static guint get_icon_size_for_zoom_level (NautilusGridZoomLevel zoom_level);
-
 #define get_view_item(li) \
         (NAUTILUS_VIEW_ITEM (gtk_tree_list_row_get_item (GTK_TREE_LIST_ROW (gtk_list_item_get_item (li)))))
 
+static const NautilusViewInfo grid_view_info =
+{
+    .view_id = NAUTILUS_VIEW_GRID_ID,
+    .zoom_level_min = NAUTILUS_GRID_ZOOM_LEVEL_SMALL,
+    .zoom_level_max = NAUTILUS_GRID_ZOOM_LEVEL_EXTRA_LARGE,
+    .zoom_level_standard = NAUTILUS_GRID_ZOOM_LEVEL_MEDIUM,
+};
+
+static NautilusViewInfo
+real_get_view_info (NautilusListBase *list_base)
+{
+    return grid_view_info;
+}
+
 static gint
 nautilus_grid_view_sort (gconstpointer a,
                          gconstpointer b,
@@ -53,31 +64,36 @@ nautilus_grid_view_sort (gconstpointer a,
 }
 
 static void
-real_begin_loading (NautilusFilesView *files_view)
+update_sort_directories_first (NautilusGridView *self)
 {
-    NautilusGridView *self = NAUTILUS_GRID_VIEW (files_view);
+    NautilusFile *directory_as_file = nautilus_list_base_get_directory_as_file (NAUTILUS_LIST_BASE (self));
+    NautilusViewModel *model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
 
-    NAUTILUS_FILES_VIEW_CLASS (nautilus_grid_view_parent_class)->begin_loading (files_view);
+    if (nautilus_file_is_in_search (directory_as_file))
+    {
+        self->directories_first = FALSE;
+    }
+    else
+    {
+        self->directories_first = g_settings_get_boolean (gtk_filechooser_preferences,
+                                                          NAUTILUS_PREFERENCES_SORT_DIRECTORIES_FIRST);
+    }
 
-    self->directories_first = nautilus_files_view_should_sort_directories_first (NAUTILUS_FILES_VIEW (self));
+    if (model != NULL)
+    {
+        nautilus_view_model_sort (model);
+    }
 }
 
 static void
-real_bump_zoom_level (NautilusFilesView *files_view,
-                      int                zoom_increment)
+nautilus_grid_view_setup_directory (NautilusListBase  *list_base,
+                                    NautilusDirectory *new_directory)
 {
-    NautilusGridView *self = NAUTILUS_GRID_VIEW (files_view);
-    NautilusGridZoomLevel new_level;
+    NautilusGridView *self = NAUTILUS_GRID_VIEW (list_base);
 
-    new_level = self->zoom_level + zoom_increment;
+    NAUTILUS_LIST_BASE_CLASS (nautilus_grid_view_parent_class)->setup_directory (list_base, new_directory);
 
-    if (new_level >= NAUTILUS_GRID_ZOOM_LEVEL_SMALL &&
-        new_level <= NAUTILUS_GRID_ZOOM_LEVEL_EXTRA_LARGE)
-    {
-        g_action_group_change_action_state (self->action_group,
-                                            "zoom-to-level",
-                                            g_variant_new_int32 (new_level));
-    }
+    update_sort_directories_first (self);
 }
 
 static guint
@@ -121,15 +137,13 @@ get_icon_size_for_zoom_level (NautilusGridZoomLevel zoom_level)
 static gint
 get_default_zoom_level (void)
 {
-    NautilusGridZoomLevel default_zoom_level;
-
-    default_zoom_level = g_settings_get_enum (nautilus_icon_view_preferences,
-                                              NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL);
+    int default_zoom_level = g_settings_get_enum (nautilus_icon_view_preferences,
+                                                  NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL);
 
     /* Sanitize preference value */
     return CLAMP (default_zoom_level,
-                  NAUTILUS_GRID_ZOOM_LEVEL_SMALL,
-                  NAUTILUS_GRID_ZOOM_LEVEL_EXTRA_LARGE);
+                  grid_view_info.zoom_level_min,
+                  grid_view_info.zoom_level_max);
 }
 
 static void
@@ -142,7 +156,7 @@ set_captions_from_preferences (NautilusGridView *self)
                                  NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS);
 
     /* Set a celling on the number of captions depending on the zoom level. */
-    n_captions_for_zoom_level = MIN (1 + self->zoom_level,
+    n_captions_for_zoom_level = MIN ((uint) self->zoom_level + 1,
                                      G_N_ELEMENTS (self->caption_attributes));
 
     /* Reset array to zeros beforehand, as we may not refill all elements. */
@@ -164,67 +178,39 @@ set_captions_from_preferences (NautilusGridView *self)
 }
 
 static void
-set_zoom_level (NautilusGridView *self,
-                guint             new_level)
+real_set_zoom_level (NautilusListBase *list_base,
+                     int               new_level)
 {
+    NautilusGridView *self = NAUTILUS_GRID_VIEW (list_base);
+
+    g_return_if_fail (new_level >= grid_view_info.zoom_level_min &&
+                      new_level <= grid_view_info.zoom_level_max);
+
     self->zoom_level = new_level;
 
+    if (g_settings_get_enum (nautilus_icon_view_preferences,
+                             NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL) != new_level)
+    {
+        g_settings_set_enum (nautilus_icon_view_preferences,
+                             NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
+                             new_level);
+    }
+
     /* The zoom level may change how many captions are allowed. Update it before
      * notifying the icon size change, under the assumption that NautilusGridCell
      * updates captions whenever the icon size is set*/
     set_captions_from_preferences (self);
 
     g_object_notify (G_OBJECT (self), "icon-size");
-
-    nautilus_files_view_update_toolbar_menus (NAUTILUS_FILES_VIEW (self));
-}
-
-static void
-real_restore_standard_zoom_level (NautilusFilesView *files_view)
-{
-    NautilusGridView *self;
-
-    self = NAUTILUS_GRID_VIEW (files_view);
-    g_action_group_change_action_state (self->action_group,
-                                        "zoom-to-level",
-                                        g_variant_new_int32 (NAUTILUS_GRID_ZOOM_LEVEL_MEDIUM));
-}
-
-static gboolean
-real_is_zoom_level_default (NautilusFilesView *files_view)
-{
-    NautilusGridView *self;
-    guint icon_size;
-
-    self = NAUTILUS_GRID_VIEW (files_view);
-    icon_size = get_icon_size_for_zoom_level (self->zoom_level);
-
-    return icon_size == NAUTILUS_GRID_ICON_SIZE_MEDIUM;
-}
-
-static gboolean
-real_can_zoom_in (NautilusFilesView *files_view)
-{
-    NautilusGridView *self = NAUTILUS_GRID_VIEW (files_view);
-
-    return self->zoom_level < NAUTILUS_GRID_ZOOM_LEVEL_EXTRA_LARGE;
-}
-
-static gboolean
-real_can_zoom_out (NautilusFilesView *files_view)
-{
-    NautilusGridView *self = NAUTILUS_GRID_VIEW (files_view);
-
-    return self->zoom_level > NAUTILUS_GRID_ZOOM_LEVEL_SMALL;
 }
 
 /* The generic implementation in src/nautilus-list-base.c doesn't allow the
  * 2-dimensional movements expected from a grid. Let's hack GTK here. */
 static void
-real_preview_selection_event (NautilusFilesView *files_view,
-                              GtkDirectionType   direction)
+real_preview_selection_event (NautilusListBase *list_base,
+                              GtkDirectionType  direction)
 {
-    NautilusGridView *self = NAUTILUS_GRID_VIEW (files_view);
+    NautilusGridView *self = NAUTILUS_GRID_VIEW (list_base);
     guint direction_keyval;
     g_autoptr (GtkShortcutTrigger) direction_trigger = NULL;
     g_autoptr (GListModel) controllers = NULL;
@@ -303,7 +289,7 @@ real_preview_selection_event (NautilusFilesView *files_view,
     /* If the hack fails (GTK may change it's internal behavior), fallback. */
     if (!success)
     {
-        NAUTILUS_FILES_VIEW_CLASS (nautilus_grid_view_parent_class)->preview_selection_event (files_view, direction);
+        NAUTILUS_LIST_BASE_CLASS (nautilus_grid_view_parent_class)->preview_selection_event (list_base, direction);
     }
 }
 
@@ -322,7 +308,7 @@ on_grid_view_item_activated (GtkGridView *grid_view,
 {
     NautilusGridView *self = NAUTILUS_GRID_VIEW (user_data);
 
-    nautilus_files_view_activate_selection (NAUTILUS_FILES_VIEW (self));
+    nautilus_list_base_activate_selection (NAUTILUS_LIST_BASE (self), FALSE);
 }
 
 static guint
@@ -341,88 +327,52 @@ real_get_view_ui (NautilusListBase *list_base_view)
     return GTK_WIDGET (self->view_ui);
 }
 
-static void
-real_scroll_to_item (NautilusListBase *list_base_view,
-                     guint             position)
+static int
+real_get_zoom_level (NautilusListBase *list_base_view)
 {
     NautilusGridView *self = NAUTILUS_GRID_VIEW (list_base_view);
 
-    gtk_widget_activate_action (GTK_WIDGET (self->view_ui),
-                                "list.scroll-to-item",
-                                "u",
-                                position);
+    return self->zoom_level;
 }
 
 static void
-real_sort_directories_first_changed (NautilusFilesView *files_view)
+real_scroll_to (NautilusListBase   *list_base_view,
+                guint               position,
+                GtkListScrollFlags  flags,
+                GtkScrollInfo      *scroll)
 {
-    NautilusGridView *self;
-    NautilusViewModel *model;
-    g_autoptr (GtkCustomSorter) sorter = NULL;
+    NautilusGridView *self = NAUTILUS_GRID_VIEW (list_base_view);
 
-    self = NAUTILUS_GRID_VIEW (files_view);
-    self->directories_first = nautilus_files_view_should_sort_directories_first (NAUTILUS_FILES_VIEW (self));
+    gtk_grid_view_scroll_to (self->view_ui, position, flags, scroll);
+}
 
-    model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
-    sorter = gtk_custom_sorter_new (nautilus_grid_view_sort, self, NULL);
-    nautilus_view_model_set_sorter (model, GTK_SORTER (sorter));
+static GVariant *
+real_get_sort_state (NautilusListBase *list_base)
+{
+    NautilusGridView *self = NAUTILUS_GRID_VIEW (list_base);
+
+    return g_variant_take_ref (g_variant_new ("(sb)",
+                                              g_quark_to_string (self->sort_attribute),
+                                              self->reversed));
 }
 
 static void
-action_sort_order_changed (GSimpleAction *action,
-                           GVariant      *value,
-                           gpointer       user_data)
+real_set_sort_state (NautilusListBase *list_base,
+                     GVariant         *value)
 {
-    g_autoptr (GVariant) old_value = g_action_get_state (G_ACTION (action));
+    NautilusGridView *self = NAUTILUS_GRID_VIEW (list_base);
     const gchar *target_name;
-    NautilusGridView *self = NAUTILUS_GRID_VIEW (user_data);
-    NautilusViewModel *model;
+    NautilusViewModel *model = nautilus_list_base_get_model (list_base);
     g_autoptr (GtkCustomSorter) sorter = NULL;
 
-    /* Don't resort if the action is in the same state as before */
-    if (g_variant_equal (value, old_value))
-    {
-        return;
-    }
+    /* Sort state binding should be set after the model is set.*/
+    g_return_if_fail (model != NULL);
 
     g_variant_get (value, "(&sb)", &target_name, &self->reversed);
     self->sort_attribute = g_quark_from_string (target_name);
 
     sorter = gtk_custom_sorter_new (nautilus_grid_view_sort, self, NULL);
-    model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
     nautilus_view_model_set_sorter (model, GTK_SORTER (sorter));
-    set_directory_sort_metadata (nautilus_files_view_get_directory_as_file (NAUTILUS_FILES_VIEW (self)),
-                                 target_name,
-                                 self->reversed);
-
-    g_simple_action_set_state (action, value);
-}
-
-static guint
-real_get_view_id (NautilusFilesView *files_view)
-{
-    return NAUTILUS_VIEW_GRID_ID;
-}
-
-static void
-action_zoom_to_level (GSimpleAction *action,
-                      GVariant      *state,
-                      gpointer       user_data)
-{
-    NautilusGridView *self = NAUTILUS_GRID_VIEW (user_data);
-    int zoom_level;
-
-    zoom_level = g_variant_get_int32 (state);
-    set_zoom_level (self, zoom_level);
-    g_simple_action_set_state (G_SIMPLE_ACTION (action), state);
-
-    if (g_settings_get_enum (nautilus_icon_view_preferences,
-                             NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL) != zoom_level)
-    {
-        g_settings_set_enum (nautilus_icon_view_preferences,
-                             NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
-                             zoom_level);
-    }
 }
 
 static void
@@ -471,10 +421,6 @@ bind_cell (GtkSignalListItemFactory *factory,
         parent = gtk_widget_get_parent (cell);
         gtk_widget_set_halign (parent, GTK_ALIGN_CENTER);
         gtk_widget_set_valign (parent, GTK_ALIGN_START);
-
-        gtk_accessible_update_relation (GTK_ACCESSIBLE (parent),
-                                        GTK_ACCESSIBLE_RELATION_LABELLED_BY, cell, NULL,
-                                        -1);
     }
 }
 
@@ -501,6 +447,7 @@ setup_cell (GtkSignalListItemFactory *factory,
 {
     NautilusGridView *self = NAUTILUS_GRID_VIEW (user_data);
     NautilusGridCell *cell;
+    GtkExpression *expression;
 
     cell = nautilus_grid_cell_new (NAUTILUS_LIST_BASE (self));
     gtk_list_item_set_child (listitem, GTK_WIDGET (cell));
@@ -512,23 +459,29 @@ setup_cell (GtkSignalListItemFactory *factory,
                             G_BINDING_SYNC_CREATE);
 
     nautilus_grid_cell_set_caption_attributes (cell, self->caption_attributes);
+
+    /* Use file display name as accessible label. Explaining in pseudo-code:
+     * listitem:accessible-name :- listitem:item:item:file:display-name */
+    expression = gtk_property_expression_new (GTK_TYPE_LIST_ITEM, NULL, "item");
+    expression = gtk_property_expression_new (GTK_TYPE_TREE_LIST_ROW, expression, "item");
+    expression = gtk_property_expression_new (NAUTILUS_TYPE_VIEW_ITEM, expression, "file");
+    expression = gtk_property_expression_new (NAUTILUS_TYPE_FILE, expression, "display-name");
+    gtk_expression_bind (expression, listitem, "accessible-label", listitem);
 }
 
 static GtkGridView *
 create_view_ui (NautilusGridView *self)
 {
-    NautilusViewModel *model;
     GtkListItemFactory *factory;
     GtkWidget *widget;
 
-    model = nautilus_list_base_get_model (NAUTILUS_LIST_BASE (self));
-
     factory = gtk_signal_list_item_factory_new ();
     g_signal_connect (factory, "setup", G_CALLBACK (setup_cell), self);
     g_signal_connect (factory, "bind", G_CALLBACK (bind_cell), self);
     g_signal_connect (factory, "unbind", G_CALLBACK (unbind_cell), self);
 
-    widget = gtk_grid_view_new (GTK_SELECTION_MODEL (model), factory);
+    widget = gtk_grid_view_new (NULL, factory);
+    g_object_bind_property (self, "model", widget, "model", G_BINDING_SYNC_CREATE);
 
     /* We don't use the built-in child activation feature for clicks because it
      * doesn't fill all our needs nor does it match our expected behavior.
@@ -549,41 +502,31 @@ create_view_ui (NautilusGridView *self)
     return GTK_GRID_VIEW (widget);
 }
 
-const GActionEntry view_icon_actions[] =
-{
-    { "sort", NULL, "(sb)", "('invalid',false)", action_sort_order_changed },
-    { "zoom-to-level", NULL, NULL, "100", action_zoom_to_level }
-};
-
 static void
 nautilus_grid_view_class_init (NautilusGridViewClass *klass)
 {
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
-    NautilusFilesViewClass *files_view_class = NAUTILUS_FILES_VIEW_CLASS (klass);
     NautilusListBaseClass *list_base_view_class = NAUTILUS_LIST_BASE_CLASS (klass);
 
     object_class->dispose = dispose;
     object_class->finalize = finalize;
 
-    files_view_class->begin_loading = real_begin_loading;
-    files_view_class->bump_zoom_level = real_bump_zoom_level;
-    files_view_class->can_zoom_in = real_can_zoom_in;
-    files_view_class->can_zoom_out = real_can_zoom_out;
-    files_view_class->sort_directories_first_changed = real_sort_directories_first_changed;
-    files_view_class->get_view_id = real_get_view_id;
-    files_view_class->restore_standard_zoom_level = real_restore_standard_zoom_level;
-    files_view_class->is_zoom_level_default = real_is_zoom_level_default;
-    files_view_class->preview_selection_event = real_preview_selection_event;
-
     list_base_view_class->get_icon_size = real_get_icon_size;
+    list_base_view_class->get_sort_state = real_get_sort_state;
+    list_base_view_class->get_view_info = real_get_view_info;
     list_base_view_class->get_view_ui = real_get_view_ui;
-    list_base_view_class->scroll_to_item = real_scroll_to_item;
+    list_base_view_class->get_zoom_level = real_get_zoom_level;
+    list_base_view_class->preview_selection_event = real_preview_selection_event;
+    list_base_view_class->scroll_to = real_scroll_to;
+    list_base_view_class->set_sort_state = real_set_sort_state;
+    list_base_view_class->set_zoom_level = real_set_zoom_level;
+    list_base_view_class->setup_directory = nautilus_grid_view_setup_directory;
 }
 
 static void
 nautilus_grid_view_init (NautilusGridView *self)
 {
-    GtkWidget *content_widget;
+    GtkWidget *scrolled_window = nautilus_list_base_get_scrolled_window (NAUTILUS_LIST_BASE (self));
 
     gtk_widget_add_css_class (GTK_WIDGET (self), "nautilus-grid-view");
 
@@ -594,32 +537,24 @@ nautilus_grid_view_init (NautilusGridView *self)
                              self,
                              G_CONNECT_SWAPPED);
 
-    content_widget = nautilus_files_view_get_content_widget (NAUTILUS_FILES_VIEW (self));
 
     self->view_ui = create_view_ui (self);
     nautilus_list_base_setup_gestures (NAUTILUS_LIST_BASE (self));
 
-    gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (content_widget),
+    gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_WIDGET (self->view_ui));
 
-    self->action_group = nautilus_files_view_get_action_group (NAUTILUS_FILES_VIEW (self));
-    g_action_map_add_action_entries (G_ACTION_MAP (self->action_group),
-                                     view_icon_actions,
-                                     G_N_ELEMENTS (view_icon_actions),
-                                     self);
+    g_signal_connect_object (gtk_filechooser_preferences,
+                             "changed::" NAUTILUS_PREFERENCES_SORT_DIRECTORIES_FIRST,
+                             G_CALLBACK (update_sort_directories_first),
+                             self,
+                             G_CONNECT_SWAPPED);
 
-    self->directories_first = nautilus_files_view_should_sort_directories_first (NAUTILUS_FILES_VIEW (self));
-
-    self->zoom_level = get_default_zoom_level ();
-    /* Keep the action synced with the actual value, so the toolbar can poll it */
-    g_action_group_change_action_state (nautilus_files_view_get_action_group (NAUTILUS_FILES_VIEW (self)),
-                                        "zoom-to-level", g_variant_new_int32 (self->zoom_level));
+    nautilus_list_base_set_zoom_level (NAUTILUS_LIST_BASE (self), get_default_zoom_level ());
 }
 
 NautilusGridView *
-nautilus_grid_view_new (NautilusWindowSlot *slot)
+nautilus_grid_view_new (void)
 {
-    return g_object_new (NAUTILUS_TYPE_GRID_VIEW,
-                         "window-slot", slot,
-                         NULL);
+    return g_object_new (NAUTILUS_TYPE_GRID_VIEW, NULL);
 }
