@@ -24,7 +24,7 @@ struct _NautilusIconInfo
     GObject parent;
 
     gboolean sole_owner;
-    gint64 last_use_time;
+    guint64 last_use_time;
     GdkPaintable *paintable;
 
     char *icon_name;
@@ -166,7 +166,7 @@ static guint reap_cache_timeout = 0;
 
 #define MICROSEC_PER_SEC ((guint64) 1000000L)
 
-static guint time_now;
+static guint64 time_now;
 
 static gboolean
 reap_old_icon (gpointer key,
@@ -395,13 +395,14 @@ nautilus_icon_info_lookup (GIcon *icon,
         return g_object_ref (icon_info);
     }
 
-    icon_paintable = gtk_icon_theme_lookup_by_gicon (gtk_icon_theme_get_for_display (gdk_display_get_default ()),
-                                                     icon, size, scale, GTK_TEXT_DIR_NONE, 0);
-    if (icon_paintable == NULL)
+    GtkIconTheme *theme = gtk_icon_theme_get_for_display (gdk_display_get_default ());
+    if (!gtk_icon_theme_has_gicon (theme, icon))
     {
         return nautilus_icon_info_new_for_paintable (NULL, scale);
     }
 
+    icon_paintable = gtk_icon_theme_lookup_by_gicon (theme, icon, size, scale, GTK_TEXT_DIR_NONE, 0);
+
     if (G_IS_THEMED_ICON (icon))
     {
         ThemedIconKey lookup_key;
