@@ -17,20 +17,18 @@
 G_BEGIN_DECLS
 
 /* Methods */
+void               nautilus_list_base_activate_selection (NautilusListBase *self,
+                                                          gboolean          open_in_new_tab);
+NautilusFile      *nautilus_list_base_get_directory_as_file (NautilusListBase *self);
 NautilusViewModel *nautilus_list_base_get_model     (NautilusListBase *self);
+GtkWidget         *nautilus_list_base_get_scrolled_window (NautilusListBase *self);
 void               nautilus_list_base_setup_gestures (NautilusListBase *self);
 
 /* Shareable helpers */
-void                          set_directory_sort_metadata       (NautilusFile *file,
-                                                                 const gchar  *sort_attribute,
-                                                                 gboolean      reversed);
 void                          setup_cell_common                 (GObject          *listitem,
                                                                  NautilusViewCell *cell);
 void                          setup_cell_hover                  (NautilusViewCell *cell);
 void                          setup_cell_hover_inner_target     (NautilusViewCell *cell,
                                                                  GtkWidget        *target);
 
-void                          set_focus_item                    (NautilusListBase *self,
-                                                                 NautilusViewItem *item);
-
 G_END_DECLS
