@@ -18,31 +18,26 @@
  *
  *  Authors: Maciej Stachowiak <mjs@eazel.com>
  */
+#define G_LOG_DOMAIN "nautilus-mime"
 
 #include "nautilus-mime-actions.h"
 
 #include <eel/eel-stock-dialogs.h>
-#include <eel/eel-string.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <string.h>
 
-#define DEBUG_FLAG NAUTILUS_DEBUG_MIME
-#include "nautilus-debug.h"
-
 #include "nautilus-application.h"
 #include "nautilus-enums.h"
 #include "nautilus-file.h"
 #include "nautilus-file-utilities.h"
 #include "nautilus-file-operations.h"
 #include "nautilus-global-preferences.h"
-#include "nautilus-metadata.h"
 #include "nautilus-program-choosing.h"
+#include "nautilus-scheme.h"
 #include "nautilus-signaller.h"
 #include "nautilus-ui-utilities.h"
-#include "nautilus-window.h"
-#include "nautilus-window-slot.h"
 
 typedef enum
 {
@@ -91,12 +86,6 @@ typedef struct
     GList *unhandled_open_in_app_uris;
 } ActivateParameters;
 
-typedef struct
-{
-    ActivateParameters *activation_params;
-    GQueue *uris;
-} ApplicationLaunchAsyncParameters;
-
 /* Microsoft mime types at https://blogs.msdn.microsoft.com/vsofficedeveloper/2008/05/08/office-2007-file-format-mime-types-for-http-content-streaming-2/ */
 struct
 {
@@ -110,122 +99,165 @@ struct
     },
     {
         N_("Files"),
-        { "application/octet-stream",
-          "text/plain",
-          NULL}
+        {
+            "application/octet-stream",
+            "text/plain",
+            "text/markdown",
+            NULL
+        }
     },
     {
         N_("Folders"),
-        { "inode/directory",
-          NULL}
+        {
+            "inode/directory",
+            NULL
+        }
     },
-    { N_("Documents"),
-      { "application/rtf",
-        "application/msword",
-        "application/vnd.sun.xml.writer",
-        "application/vnd.sun.xml.writer.global",
-        "application/vnd.sun.xml.writer.template",
-        "application/vnd.oasis.opendocument.text",
-        "application/vnd.oasis.opendocument.text-template",
-        "application/x-abiword",
-        "application/x-applix-word",
-        "application/x-mswrite",
-        "application/docbook+xml",
-        "application/x-kword",
-        "application/x-kword-crypt",
-        "application/x-lyx",
-        "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
-        NULL}},
-    { N_("Illustration"),
-      { "application/illustrator",
-        "application/vnd.corel-draw",
-        "application/vnd.stardivision.draw",
-        "application/vnd.oasis.opendocument.graphics",
-        "application/x-dia-diagram",
-        "application/x-karbon",
-        "application/x-killustrator",
-        "application/x-kivio",
-        "application/x-kontour",
-        "application/x-wpg",
-        NULL}},
-    { N_("Music"),
-      { "application/ogg",
-        "audio/x-vorbis+ogg",
-        "audio/ac3",
-        "audio/basic",
-        "audio/midi",
-        "audio/x-flac",
-        "audio/mp4",
-        "audio/mpeg",
-        "audio/x-mpeg",
-        "audio/x-ms-asx",
-        "audio/x-pn-realaudio",
-        NULL}},
-    { N_("PDF / PostScript"),
-      { "application/pdf",
-        "application/postscript",
-        "application/x-dvi",
-        "image/x-eps",
-        "image/vnd.djvu+multipage",
-        NULL}},
-    { N_("Picture"),
-      { "application/vnd.oasis.opendocument.image",
-        "application/x-krita",
-        "image/bmp",
-        "image/cgm",
-        "image/gif",
-        "image/jpeg",
-        "image/jpeg2000",
-        "image/png",
-        "image/svg+xml",
-        "image/tiff",
-        "image/x-compressed-xcf",
-        "image/x-pcx",
-        "image/x-photo-cd",
-        "image/x-psd",
-        "image/x-tga",
-        "image/x-xcf",
-        NULL}},
-    { N_("Presentation"),
-      { "application/vnd.ms-powerpoint",
-        "application/vnd.sun.xml.impress",
-        "application/vnd.oasis.opendocument.presentation",
-        "application/x-magicpoint",
-        "application/x-kpresenter",
-        "application/vnd.openxmlformats-officedocument.presentationml.presentation",
-        NULL}},
-    { N_("Spreadsheet"),
-      { "application/vnd.lotus-1-2-3",
-        "application/vnd.ms-excel",
-        "application/vnd.stardivision.calc",
-        "application/vnd.sun.xml.calc",
-        "application/vnd.oasis.opendocument.spreadsheet",
-        "application/x-applix-spreadsheet",
-        "application/x-gnumeric",
-        "application/x-kspread",
-        "application/x-kspread-crypt",
-        "application/x-quattropro",
-        "application/x-sc",
-        "application/x-siag",
-        "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
-        NULL}},
-    { N_("Text File"),
-      { "text/plain",
-        NULL}},
-    { N_("Video"),
-      { "video/mp4",
-        "video/3gpp",
-        "video/mpeg",
-        "video/quicktime",
-        "video/vivo",
-        "video/x-avi",
-        "video/x-mng",
-        "video/x-ms-asf",
-        "video/x-ms-wmv",
-        "video/x-msvideo",
-        "video/x-nsv",
-        "video/x-real-video",
-        NULL}}
+    {
+        N_("Documents"),
+        {
+            "application/rtf",
+            "application/msword",
+            "application/vnd.sun.xml.writer",
+            "application/vnd.sun.xml.writer.global",
+            "application/vnd.sun.xml.writer.template",
+            "application/vnd.oasis.opendocument.text",
+            "application/vnd.oasis.opendocument.text-template",
+            "application/x-abiword",
+            "application/x-applix-word",
+            "application/x-mswrite",
+            "application/docbook+xml",
+            "application/x-kword",
+            "application/x-kword-crypt",
+            "application/x-lyx",
+            "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
+            "text/markdown",
+            NULL
+        }
+    },
+    {
+        N_("Illustration"),
+        {
+            "application/illustrator",
+            "application/vnd.corel-draw",
+            "application/vnd.stardivision.draw",
+            "application/vnd.oasis.opendocument.graphics",
+            "application/x-dia-diagram",
+            "application/x-karbon",
+            "application/x-killustrator",
+            "application/x-kivio",
+            "application/x-kontour",
+            "application/x-wpg",
+            NULL
+        }
+    },
+    {
+        N_("Music"),
+        {
+            "application/ogg",
+            "audio/x-vorbis+ogg",
+            "audio/ac3",
+            "audio/basic",
+            "audio/midi",
+            "audio/x-flac",
+            "audio/mp4",
+            "audio/mpeg",
+            "audio/x-mpeg",
+            "audio/x-ms-asx",
+            "audio/x-pn-realaudio",
+            NULL
+        }
+    },
+    {
+        N_("PDF / PostScript"),
+        {
+            "application/pdf",
+            "application/postscript",
+            "application/x-dvi",
+            "image/x-eps",
+            "image/vnd.djvu+multipage",
+            NULL
+        }
+    },
+    {
+        N_("Picture"),
+        {
+            "application/vnd.oasis.opendocument.image",
+            "application/x-krita",
+            "image/bmp",
+            "image/cgm",
+            "image/gif",
+            "image/jpeg",
+            "image/jpeg2000",
+            "image/png",
+            "image/svg+xml",
+            "image/tiff",
+            "image/x-compressed-xcf",
+            "image/x-pcx",
+            "image/x-photo-cd",
+            "image/x-psd",
+            "image/x-tga",
+            "image/x-xcf",
+            NULL
+        }
+    },
+    {
+        N_("Presentation"),
+        {
+            "application/vnd.ms-powerpoint",
+            "application/vnd.sun.xml.impress",
+            "application/vnd.oasis.opendocument.presentation",
+            "application/x-magicpoint",
+            "application/x-kpresenter",
+            "application/vnd.openxmlformats-officedocument.presentationml.presentation",
+            NULL
+        }
+    },
+    {
+        N_("Spreadsheet"),
+        {
+            "application/vnd.lotus-1-2-3",
+            "application/vnd.ms-excel",
+            "application/vnd.stardivision.calc",
+            "application/vnd.sun.xml.calc",
+            "application/vnd.oasis.opendocument.spreadsheet",
+            "application/x-applix-spreadsheet",
+            "application/x-gnumeric",
+            "application/x-kspread",
+            "application/x-kspread-crypt",
+            "application/x-quattropro",
+            "application/x-sc",
+            "application/x-siag",
+            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
+            NULL
+        }
+    },
+    {
+        N_("Text File"),
+        {
+            "text/plain",
+            "text/markdown",
+            NULL
+        }
+    },
+    {
+        N_("Video"),
+        {
+            "video/mp4",
+            "video/3gpp",
+            "video/mpeg",
+            "video/quicktime",
+            "video/vivo",
+            "video/x-avi",
+            "video/x-mng",
+            "video/x-ms-asf",
+            "video/x-ms-wmv",
+            "video/x-msvideo",
+            "video/x-nsv",
+            "video/x-real-video",
+            NULL
+        }
+    }
 };
 
 /* Number of seconds until cancel dialog shows up */
@@ -394,7 +426,6 @@ GAppInfo *
 nautilus_mime_get_default_application_for_file (NautilusFile *file)
 {
     GAppInfo *app;
-    char *mime_type;
     char *uri_scheme;
 
     if (!nautilus_mime_actions_check_if_required_attributes_ready (file))
@@ -402,10 +433,8 @@ nautilus_mime_get_default_application_for_file (NautilusFile *file)
         return NULL;
     }
 
-    mime_type = nautilus_file_get_mime_type (file);
-    app = g_app_info_get_default_for_type (mime_type,
+    app = g_app_info_get_default_for_type (nautilus_file_get_mime_type (file),
                                            !nautilus_file_has_local_path (file));
-    g_free (mime_type);
 
     if (app == NULL)
     {
@@ -424,18 +453,8 @@ static int
 file_compare_by_mime_type (NautilusFile *file_a,
                            NautilusFile *file_b)
 {
-    char *mime_type_a, *mime_type_b;
-    int ret;
-
-    mime_type_a = nautilus_file_get_mime_type (file_a);
-    mime_type_b = nautilus_file_get_mime_type (file_b);
-
-    ret = strcmp (mime_type_a, mime_type_b);
-
-    g_free (mime_type_a);
-    g_free (mime_type_b);
-
-    return ret;
+    return strcmp (nautilus_file_get_mime_type (file_a),
+                   nautilus_file_get_mime_type (file_b));
 }
 
 static int
@@ -461,13 +480,20 @@ nautilus_mime_get_default_application_for_files (GList *files)
 {
     GList *l, *sorted_files;
     NautilusFile *file;
-    GAppInfo *app, *one_app;
+    GAppInfo *app = NULL;
 
     g_assert (files != NULL);
 
+    if (nautilus_application_is_sandboxed ())
+    {
+        /* While running as a flatpak, we don't know the default app. The code
+         * bellow might even get a wrong result because it may find matches
+         * inside the sandbox which are incorrect. */
+        return NULL;
+    }
+
     sorted_files = g_list_sort (g_list_copy (files), (GCompareFunc) file_compare_by_mime_type);
 
-    app = NULL;
     for (l = sorted_files; l != NULL; l = l->next)
     {
         file = l->data;
@@ -479,28 +505,17 @@ nautilus_mime_get_default_application_for_files (GList *files)
             continue;
         }
 
-        one_app = nautilus_mime_get_default_application_for_file (file);
+        g_autoptr (GAppInfo) one_app = nautilus_mime_get_default_application_for_file (file);
+
         if (one_app == NULL || (app != NULL && !g_app_info_equal (app, one_app)))
         {
-            if (app)
-            {
-                g_object_unref (app);
-            }
-            if (one_app)
-            {
-                g_object_unref (one_app);
-            }
-            app = NULL;
+            g_clear_object (&app);
             break;
         }
 
         if (app == NULL)
         {
-            app = one_app;
-        }
-        else
-        {
-            g_object_unref (one_app);
+            app = g_steal_pointer (&one_app);
         }
     }
 
@@ -562,8 +577,7 @@ static void
 report_broken_symbolic_link (GtkWindow    *parent_window,
                              NautilusFile *file)
 {
-    char *target_path;
-    char *display_name;
+    const char *target_path;
     char *detail;
     GtkWidget *dialog;
     TrashBrokenSymbolicLinkData *data;
@@ -572,7 +586,6 @@ report_broken_symbolic_link (GtkWindow    *parent_window,
 
     g_assert (nautilus_file_is_broken_symbolic_link (file));
 
-    display_name = nautilus_file_get_display_name (file);
     can_trash = nautilus_file_can_trash (file) && !nautilus_file_is_in_trash (file);
 
     target_path = nautilus_file_get_symbolic_link_target_path (file);
@@ -591,7 +604,7 @@ report_broken_symbolic_link (GtkWindow    *parent_window,
         dialog = adw_message_dialog_new (parent_window, NULL, detail);
         adw_message_dialog_format_heading (ADW_MESSAGE_DIALOG (dialog),
                                            _("The link “%s” is broken. Move it to Trash?"),
-                                           display_name);
+                                           nautilus_file_get_display_name (file));
         adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
                                           "cancel", _("_Cancel"),
                                           "move-to-trash", _("Mo_ve to Trash"),
@@ -602,11 +615,10 @@ report_broken_symbolic_link (GtkWindow    *parent_window,
         dialog = adw_message_dialog_new (parent_window, NULL, detail);
         adw_message_dialog_format_heading (ADW_MESSAGE_DIALOG (dialog),
                                            _("The link “%s” is broken."),
-                                           display_name);
+                                           nautilus_file_get_display_name (file));
         adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog),
                                          "cancel", _("Cancel"));
     }
-    g_free (display_name);
 
     adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "cancel");
 
@@ -628,7 +640,6 @@ report_broken_symbolic_link (GtkWindow    *parent_window,
                       G_CALLBACK (trash_symbolic_link_cb),
                       data);
 
-    g_free (target_path);
     g_free (detail);
 }
 
@@ -890,15 +901,6 @@ activation_parameters_free (ActivateParameters *parameters)
     g_free (parameters);
 }
 
-static void
-application_launch_async_parameters_free (ApplicationLaunchAsyncParameters *parameters)
-{
-    g_queue_free (parameters->uris);
-    activation_parameters_free (parameters->activation_params);
-
-    g_free (parameters);
-}
-
 static void
 cancel_activate_callback (gpointer callback_data)
 {
@@ -1035,7 +1037,6 @@ typedef struct
     NautilusOpenFlags flags;
     char *activation_directory;
     gboolean user_confirmation;
-    char *uri;
     GDBusProxy *proxy;
     GtkWidget *dialog;
 } ActivateParametersInstall;
@@ -1060,28 +1061,23 @@ activate_parameters_install_free (ActivateParametersInstall *parameters_install)
     nautilus_file_unref (parameters_install->file);
     nautilus_file_list_free (parameters_install->files);
     g_free (parameters_install->activation_directory);
-    g_free (parameters_install->uri);
     g_free (parameters_install);
 }
 
 static char *
-get_application_no_mime_type_handler_message (NautilusFile *file,
-                                              char         *uri)
+get_application_no_mime_type_handler_message (NautilusFile *file)
 {
     char *uri_for_display;
-    char *name;
     char *error_message;
 
-    name = nautilus_file_get_display_name (file);
-
     /* Truncate the URI so it doesn't get insanely wide. Note that even
      * though the dialog uses wrapped text, if the URI doesn't contain
      * white space then the text-wrapping code is too stupid to wrap it.
      */
-    uri_for_display = eel_str_middle_truncate (name, MAX_URI_IN_DIALOG_LENGTH);
+    uri_for_display = g_utf8_truncate_middle (nautilus_file_get_display_name (file),
+                                              MAX_URI_IN_DIALOG_LENGTH);
     error_message = g_strdup_printf (_("Could Not Display “%s”"), uri_for_display);
     g_free (uri_for_display);
-    g_free (name);
 
     return error_message;
 }
@@ -1173,8 +1169,8 @@ show_unhandled_type_error (ActivateParametersInstall *parameters)
     g_autofree char *body = NULL;
     g_autofree char *content_type_description = NULL;
 
-    char *mime_type = nautilus_file_get_mime_type (parameters->file);
-    char *error_message = get_application_no_mime_type_handler_message (parameters->file, parameters->uri);
+    const char *mime_type = nautilus_file_get_mime_type (parameters->file);
+    char *error_message = get_application_no_mime_type_handler_message (parameters->file);
 
     if (g_content_type_is_unknown (mime_type))
     {
@@ -1204,7 +1200,6 @@ show_unhandled_type_error (ActivateParametersInstall *parameters)
                       G_CALLBACK (choose_program), parameters);
 
     g_free (error_message);
-    g_free (mime_type);
 }
 
 static void
@@ -1273,7 +1268,7 @@ search_for_application_mime_type (ActivateParametersInstall *parameters_install,
                        (GAsyncReadyCallback) search_for_application_dbus_call_notify_cb,
                        parameters_install);
 
-    DEBUG ("InstallMimeType method invoked for %s", mime_type);
+    g_debug ("InstallMimeType method invoked for %s", mime_type);
 }
 
 static void
@@ -1281,15 +1276,13 @@ application_unhandled_file_install (GtkDialog                 *dialog,
                                     gchar                     *response,
                                     ActivateParametersInstall *parameters_install)
 {
-    char *mime_type;
-
     parameters_install->dialog = NULL;
 
     if (g_strcmp0 (response, "search-in-software") == 0)
     {
-        mime_type = nautilus_file_get_mime_type (parameters_install->file);
+        const char *mime_type = nautilus_file_get_mime_type (parameters_install->file);
+
         search_for_application_mime_type (parameters_install, mime_type);
-        g_free (mime_type);
     }
     else
     {
@@ -1304,7 +1297,8 @@ pk_proxy_appeared_cb (GObject      *source,
                       gpointer      user_data)
 {
     ActivateParametersInstall *parameters_install = user_data;
-    char *mime_type, *name_owner;
+    const char *mime_type;
+    g_autofree char *name_owner = NULL;
     char *error_message;
     GtkWidget *dialog;
     GDBusProxy *proxy;
@@ -1326,12 +1320,9 @@ pk_proxy_appeared_cb (GObject      *source,
         return;
     }
 
-    g_free (name_owner);
-
     mime_type = nautilus_file_get_mime_type (parameters_install->file);
     content_type_description = g_content_type_get_description (mime_type);
-    error_message = get_application_no_mime_type_handler_message (parameters_install->file,
-                                                                  parameters_install->uri);
+    error_message = get_application_no_mime_type_handler_message (parameters_install->file);
     /* use a custom dialog to prompt the user to install new software */
     dialog = adw_message_dialog_new (parameters_install->parent_window, error_message, NULL);
     adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dialog),
@@ -1352,7 +1343,6 @@ pk_proxy_appeared_cb (GObject      *source,
                       G_CALLBACK (application_unhandled_file_install),
                       parameters_install);
     gtk_window_present (GTK_WINDOW (dialog));
-    g_free (mime_type);
 }
 
 static void
@@ -1360,14 +1350,11 @@ application_unhandled_uri (ActivateParameters *parameters,
                            char               *uri)
 {
     gboolean show_install_mime;
-    char *mime_type;
     NautilusFile *file;
     ActivateParametersInstall *parameters_install;
 
     file = nautilus_file_get_by_uri (uri);
 
-    mime_type = nautilus_file_get_mime_type (file);
-
     /* copy the parts of parameters we are interested in as the orignal will be unref'd */
     parameters_install = g_new0 (ActivateParametersInstall, 1);
     parameters_install->slot = parameters->slot;
@@ -1382,7 +1369,6 @@ application_unhandled_uri (ActivateParameters *parameters,
     parameters_install->files = get_file_list_for_launch_locations (parameters->locations);
     parameters_install->flags = parameters->flags;
     parameters_install->user_confirmation = parameters->user_confirmation;
-    parameters_install->uri = g_strdup (uri);
 
 #ifdef ENABLE_PACKAGEKIT
     /* allow an admin to disable the PackageKit search functionality */
@@ -1392,13 +1378,11 @@ application_unhandled_uri (ActivateParameters *parameters,
     show_install_mime = FALSE;
 #endif
     /* There is no use trying to look for handlers of application/octet-stream */
-    if (g_content_type_is_unknown (mime_type))
+    if (g_content_type_is_unknown (nautilus_file_get_mime_type (file)))
     {
         show_install_mime = FALSE;
     }
 
-    g_free (mime_type);
-
     if (!show_install_mime)
     {
         goto out;
@@ -1422,36 +1406,38 @@ out:
 }
 
 static void
-launch_default_for_uris_callback (GObject      *source_object,
-                                  GAsyncResult *res,
-                                  gpointer      user_data)
+sandboxed_launcher_callback (GObject      *source_object,
+                             GAsyncResult *res,
+                             gpointer      user_data)
 {
-    ApplicationLaunchAsyncParameters *params;
-    ActivateParameters *activation_params;
-    char *uri;
+    GtkFileLauncher *launcher = GTK_FILE_LAUNCHER (source_object);
+    ActivateParameters *activation_params = (ActivateParameters *) user_data;
     g_autoptr (GError) error = NULL;
 
-    params = user_data;
-    activation_params = params->activation_params;
-    uri = g_queue_pop_head (params->uris);
-
-    nautilus_launch_default_for_uri_finish (res, &error);
+    gtk_file_launcher_launch_finish (launcher, res, &error);
     if (error == NULL)
     {
+        /* Extract URI from the launched file and add it to recents. */
+        g_autofree gchar *uri = g_file_get_uri (gtk_file_launcher_get_file (launcher));
         gtk_recent_manager_add_item (gtk_recent_manager_get_default (), uri);
     }
 
-    if (!g_queue_is_empty (params->uris))
+    if (!g_queue_is_empty (activation_params->open_in_app_uris))
     {
-        nautilus_launch_default_for_uri_async (g_queue_peek_head (params->uris),
-                                               activation_params->parent_window,
-                                               activation_params->cancellable,
-                                               launch_default_for_uris_callback,
-                                               params);
+        /* Take next file off the queue to launch it. */
+        const gchar *uri = g_queue_pop_head (activation_params->open_in_app_uris);
+        g_autoptr (GFile) location = g_file_new_for_uri (uri);
+
+        gtk_file_launcher_set_file (launcher, location);
+        gtk_file_launcher_launch (launcher,
+                                  activation_params->parent_window,
+                                  activation_params->cancellable,
+                                  sandboxed_launcher_callback,
+                                  activation_params /* callback takes ownership */);
     }
     else
     {
-        application_launch_async_parameters_free (params);
+        activation_parameters_free (activation_params);
     }
 }
 
@@ -1599,7 +1585,7 @@ activate_files_internal (ActivateParameters *parameters)
         executable_path = g_filename_from_uri (uri, NULL, NULL);
         quoted_path = g_shell_quote (executable_path);
 
-        DEBUG ("Launching file path %s", quoted_path);
+        g_debug ("Launching file path %s", quoted_path);
 
         nautilus_launch_application_from_command (display, quoted_path, FALSE, NULL);
     }
@@ -1616,7 +1602,7 @@ activate_files_internal (ActivateParameters *parameters)
         executable_path = g_filename_from_uri (uri, NULL, NULL);
         quoted_path = g_shell_quote (executable_path);
 
-        DEBUG ("Launching in terminal file quoted path %s", quoted_path);
+        g_debug ("Launching in terminal file quoted path %s", quoted_path);
 
         nautilus_launch_application_from_command (display, quoted_path, TRUE, NULL);
     }
@@ -1653,12 +1639,8 @@ activate_files_internal (ActivateParameters *parameters)
                  !nautilus_file_can_read (file) ||
                  !nautilus_file_can_execute (file)))
             {
-                g_autofree gchar *file_path = NULL;
-
                 g_free (uri);
-
-                file_path = g_file_get_path (location);
-                uri = g_strconcat ("admin://", file_path, NULL);
+                uri = g_strconcat (SCHEME_ADMIN "://", g_file_peek_path (location), NULL);
             }
 
             location_with_permissions = g_file_new_for_uri (uri);
@@ -1673,20 +1655,23 @@ activate_files_internal (ActivateParameters *parameters)
 
     if (!g_queue_is_empty (parameters->open_in_app_uris) && nautilus_application_is_sandboxed ())
     {
-        const char *uri;
-        ApplicationLaunchAsyncParameters *async_params;
+        /* Use GtkFileLauncher as a convenient wrapper for the portal.
+         *
+         * Here we won't launch all files in the queue at once in a 'for' loop,
+         * because we could get multiple portal dialogs at the same time.
+         *
+         * So, instead, we launch one at a time, waiting for the async callback
+         * to launch the next one.
+         */
+        const gchar *uri = g_queue_pop_head (parameters->open_in_app_uris);
+        g_autoptr (GFile) location = g_file_new_for_uri (uri);
+        g_autoptr (GtkFileLauncher) launcher = gtk_file_launcher_new (location);
 
-        uri = g_queue_peek_head (parameters->open_in_app_uris);
-
-        async_params = g_new0 (ApplicationLaunchAsyncParameters, 1);
-        async_params->activation_params = parameters;
-        async_params->uris = g_steal_pointer (&parameters->open_in_app_uris);
-
-        nautilus_launch_default_for_uri_async (uri,
-                                               parameters->parent_window,
-                                               parameters->cancellable,
-                                               launch_default_for_uris_callback,
-                                               async_params);
+        gtk_file_launcher_launch (launcher,
+                                  parameters->parent_window,
+                                  parameters->cancellable,
+                                  sandboxed_launcher_callback,
+                                  parameters /* callback takes ownership */);
         return;
     }
 
@@ -2186,7 +2171,6 @@ nautilus_mime_activate_files (GtkWindow          *parent_window,
                               gboolean            user_confirmation)
 {
     ActivateParameters *parameters;
-    char *file_name;
     int file_count;
     GList *l, *next;
     NautilusFile *file;
@@ -2197,7 +2181,8 @@ nautilus_mime_activate_files (GtkWindow          *parent_window,
         return;
     }
 
-    DEBUG_FILES (files, "Calling activate_files() with files:");
+    g_debug ("Calling activate_files() with files:");
+    nautilus_file_list_debug (files);
 
     parameters = g_new0 (ActivateParameters, 1);
     parameters->slot = slot;
@@ -2216,9 +2201,8 @@ nautilus_mime_activate_files (GtkWindow          *parent_window,
     file_count = g_list_length (files);
     if (file_count == 1)
     {
-        file_name = nautilus_file_get_display_name (files->data);
+        const char *file_name = nautilus_file_get_display_name (files->data);
         parameters->timed_wait_prompt = g_strdup_printf (_("Opening “%s”."), file_name);
-        g_free (file_name);
     }
     else
     {
@@ -2290,14 +2274,14 @@ nautilus_mime_activate_file (GtkWindow          *parent_window,
     g_list_free (files);
 }
 
-gint
+guint
 nautilus_mime_types_get_number_of_groups (void)
 {
     return G_N_ELEMENTS (mimetype_groups);
 }
 
 const gchar *
-nautilus_mime_types_group_get_name (gint group_index)
+nautilus_mime_types_group_get_name (guint group_index)
 {
     g_return_val_if_fail (group_index < G_N_ELEMENTS (mimetype_groups), NULL);
 
@@ -2305,7 +2289,7 @@ nautilus_mime_types_group_get_name (gint group_index)
 }
 
 GPtrArray *
-nautilus_mime_types_group_get_mimetypes (gint group_index)
+nautilus_mime_types_group_get_mimetypes (guint group_index)
 {
     GStrv group;
     GPtrArray *mimetypes;
