@@ -23,7 +23,6 @@
 #include <config.h>
 #include "nautilus-module.h"
 
-#include <eel/eel-debug.h>
 #include <gmodule.h>
 
 #define NAUTILUS_TYPE_MODULE            (nautilus_module_get_type ())
@@ -248,8 +247,8 @@ load_module_dir (const char *dirname)
     installed_module_names = g_strv_builder_end (installed_module_name_builder);
 }
 
-static void
-free_module_objects (void)
+void
+nautilus_module_teardown (void)
 {
     GList *l, *next;
 
@@ -281,8 +280,6 @@ nautilus_module_setup (void)
         initialized = TRUE;
 
         load_module_dir (NAUTILUS_EXTENSIONDIR);
-
-        eel_debug_call_at_shutdown (free_module_objects);
     }
 }
 
