@@ -24,11 +24,10 @@
 
 #pragma once
 
-#include <glib-object.h>
 #include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
-void nautilus_preferences_window_show(GtkWindow *window);
+void nautilus_preferences_window_show (GtkWindow *parent_window);
 
-G_END_DECLS
\ No newline at end of file
+G_END_DECLS
