@@ -18,6 +18,7 @@
  * Author: Alexander Larsson <alexl@redhat.com>
  *
  */
+#define G_LOG_DOMAIN "nautilus-search"
 
 #include <config.h>
 #include "nautilus-search-engine-simple.h"
@@ -25,8 +26,6 @@
 #include "nautilus-search-hit.h"
 #include "nautilus-search-provider.h"
 #include "nautilus-ui-utilities.h"
-#define DEBUG_FLAG NAUTILUS_DEBUG_SEARCH
-#include "nautilus-debug.h"
 
 #include <string.h>
 #include <glib.h>
@@ -149,11 +148,11 @@ search_thread_done (SearchThreadData *data)
 
     if (g_cancellable_is_cancelled (data->cancellable))
     {
-        DEBUG ("Simple engine finished and cancelled");
+        g_debug ("Simple engine finished and cancelled");
     }
     else
     {
-        DEBUG ("Simple engine finished");
+        g_debug ("Simple engine finished");
     }
     engine->active_search = NULL;
     nautilus_search_provider_finished (NAUTILUS_SEARCH_PROVIDER (engine),
@@ -172,7 +171,7 @@ search_thread_process_hits_idle (SearchThreadData *data,
 {
     if (!g_cancellable_is_cancelled (data->cancellable))
     {
-        DEBUG ("Simple engine add hits");
+        g_debug ("Simple engine add hits");
         nautilus_search_provider_hits_added (NAUTILUS_SEARCH_PROVIDER (data->engine),
                                              hits);
     }
@@ -353,7 +352,7 @@ visit_directory (GFile            *dir,
 
             found = FALSE;
 
-            for (gint i = 0; mime_type != NULL && i < data->mime_types->len; i++)
+            for (guint i = 0; mime_type != NULL && i < data->mime_types->len; i++)
             {
                 if (g_content_type_is_a (mime_type, g_ptr_array_index (data->mime_types, i)))
                 {
@@ -544,7 +543,7 @@ nautilus_search_engine_simple_start (NautilusSearchProvider *provider)
         return;
     }
 
-    DEBUG ("Simple engine start");
+    g_debug ("Simple engine start");
 
     data = search_thread_data_new (simple, simple->query);
 
@@ -573,7 +572,7 @@ nautilus_search_engine_simple_stop (NautilusSearchProvider *provider)
 
     if (simple->active_search != NULL)
     {
-        DEBUG ("Simple engine stop");
+        g_debug ("Simple engine stop");
         g_cancellable_cancel (simple->active_search->cancellable);
 
         if (simple->create_thread_timeout_id != 0)
