@@ -16,14 +16,13 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
+#define G_LOG_DOMAIN "nautilus-tag-manager"
 
 #include "nautilus-tag-manager.h"
 #include "nautilus-file.h"
 #include "nautilus-file-undo-operations.h"
 #include "nautilus-file-undo-manager.h"
 #include "nautilus-tracker-utilities.h"
-#define DEBUG_FLAG NAUTILUS_DEBUG_TAG_MANAGER
-#include "nautilus-debug.h"
 
 #include <gio/gunixinputstream.h>
 #include <tracker-sparql.h>
@@ -44,6 +43,8 @@ struct _NautilusTagManager
     GHashTable *starred_file_uris;
     GFile *home;
 
+    GList *pending_changed_files;
+
     GCancellable *cancellable;
 };
 
@@ -146,8 +147,6 @@ on_update_callback (GObject      *object,
 
     if (error == NULL)
     {
-        /* FIXME: make sure data->tag_manager->starred_file_uris is up to date */
-
         if (!nautilus_file_undo_manager_is_operating ())
         {
             NautilusFileUndoInfo *undo_info;
@@ -158,8 +157,6 @@ on_update_callback (GObject      *object,
             g_object_unref (undo_info);
         }
 
-        g_signal_emit_by_name (data->tag_manager, "starred-changed", nautilus_file_list_copy (data->selection));
-
         g_task_return_boolean (data->task, TRUE);
         g_object_unref (data->task);
     }
@@ -182,29 +179,30 @@ on_update_callback (GObject      *object,
  * nautilus_tag_manager_get_starred_files:
  * @self: The tag manager singleton
  *
- * Returns: (element-type gchar*) (transfer container): A list of the starred urls.
+ * Returns: (element-type gchar*) (transfer full): A list of the starred NautilusFile.
  */
 GList *
 nautilus_tag_manager_get_starred_files (NautilusTagManager *self)
 {
     GHashTableIter starred_iter;
     gchar *starred_uri;
-    GList *starred_file_uris = NULL;
+    GList *starred_files = NULL;
 
     g_hash_table_iter_init (&starred_iter, self->starred_file_uris);
     while (g_hash_table_iter_next (&starred_iter, (gpointer *) &starred_uri, NULL))
     {
-        g_autoptr (GFile) file = g_file_new_for_uri (starred_uri);
+        g_autoptr (GFile) location = g_file_new_for_uri (starred_uri);
+        NautilusFile *file = nautilus_file_get (location);
 
-        /* Skip files ouside $HOME, because we don't support starring these yet.
+        /* Skip files outside $HOME, because we don't support starring these yet.
          * See comment on nautilus_tag_manager_can_star_contents() */
-        if (g_file_has_prefix (file, self->home))
+        if (g_file_has_prefix (location, self->home))
         {
-            starred_file_uris = g_list_prepend (starred_file_uris, starred_uri);
+            starred_files = g_list_prepend (starred_files, file);
         }
     }
 
-    return starred_file_uris;
+    return starred_files;
 }
 
 static void
@@ -217,7 +215,6 @@ on_get_starred_files_cursor_callback (GObject      *object,
     const gchar *url;
     gboolean success;
     NautilusTagManager *self;
-    GList *changed_files;
     NautilusFile *file;
 
     cursor = TRACKER_SPARQL_CURSOR (object);
@@ -233,6 +230,12 @@ on_get_starred_files_cursor_callback (GObject      *object,
             g_warning ("Error on getting all tags cursor callback: %s", error->message);
         }
 
+        if (self->pending_changed_files != NULL)
+        {
+            g_signal_emit_by_name (self, "starred-changed", self->pending_changed_files);
+            g_clear_pointer (&self->pending_changed_files, nautilus_file_list_free);
+        }
+
         g_clear_object (&cursor);
         return;
     }
@@ -245,15 +248,11 @@ on_get_starred_files_cursor_callback (GObject      *object,
 
     if (file)
     {
-        changed_files = g_list_prepend (NULL, file);
-
-        g_signal_emit_by_name (self, "starred-changed", changed_files);
-
-        nautilus_file_list_free (changed_files);
+        self->pending_changed_files = g_list_prepend (self->pending_changed_files, file);
     }
     else
     {
-        DEBUG ("File %s is starred but not found", url);
+        g_debug ("File %s is starred but not found", url);
     }
 
     tracker_sparql_cursor_next_async (cursor,
@@ -386,7 +385,7 @@ nautilus_tag_manager_star_files (NautilusTagManager  *self,
     GTask *task;
     UpdateData *update_data;
 
-    DEBUG ("Starring %i files", g_list_length (selection));
+    g_debug ("Starring %i files", g_list_length (selection));
 
     task = g_task_new (object, cancellable, callback, NULL);
 
@@ -419,7 +418,7 @@ nautilus_tag_manager_unstar_files (NautilusTagManager  *self,
     GTask *task;
     UpdateData *update_data;
 
-    DEBUG ("Unstarring %i files", g_list_length (selection));
+    g_debug ("Unstarring %i files", g_list_length (selection));
 
     task = g_task_new (object, cancellable, callback, NULL);
 
@@ -450,7 +449,6 @@ on_tracker_notifier_events (TrackerNotifier *notifier,
 {
     TrackerNotifierEvent *event;
     NautilusTagManager *self;
-    int i;
     const gchar *file_url;
     GError *error = NULL;
     TrackerSparqlCursor *cursor;
@@ -461,14 +459,14 @@ on_tracker_notifier_events (TrackerNotifier *notifier,
 
     self = NAUTILUS_TAG_MANAGER (user_data);
 
-    for (i = 0; i < events->len; i++)
+    for (guint i = 0; i < events->len; i++)
     {
         event = g_ptr_array_index (events, i);
 
         file_url = tracker_notifier_event_get_urn (event);
         changed_file = NULL;
 
-        DEBUG ("Got event for file %s", file_url);
+        g_debug ("Got event for file %s", file_url);
 
         tracker_sparql_statement_bind_string (self->query_file_is_starred, "file", file_url);
         cursor = tracker_sparql_statement_execute (self->query_file_is_starred,
@@ -494,7 +492,7 @@ on_tracker_notifier_events (TrackerNotifier *notifier,
 
             if (inserted)
             {
-                DEBUG ("Added %s to starred files list", file_url);
+                g_debug ("Added %s to starred files list", file_url);
                 changed_file = nautilus_file_get_by_uri (file_url);
             }
         }
@@ -504,7 +502,7 @@ on_tracker_notifier_events (TrackerNotifier *notifier,
 
             if (removed)
             {
-                DEBUG ("Removed %s from starred files list", file_url);
+                g_debug ("Removed %s from starred files list", file_url);
                 changed_file = nautilus_file_get_by_uri (file_url);
             }
         }
@@ -542,6 +540,7 @@ nautilus_tag_manager_finalize (GObject *object)
     g_clear_object (&self->db);
     g_clear_object (&self->query_file_is_starred);
     g_clear_object (&self->query_starred_files);
+    g_clear_pointer (&self->pending_changed_files, nautilus_file_list_free);
 
     g_hash_table_destroy (self->starred_file_uris);
     g_clear_object (&self->home);
@@ -726,7 +725,6 @@ update_moved_uris_callback (GObject      *object,
                             gpointer      user_data)
 {
     g_autoptr (GError) error = NULL;
-    g_autoptr (GPtrArray) new_uris = user_data;
 
     tracker_sparql_connection_update_finish (TRACKER_SPARQL_CONNECTION (object),
                                              result,
@@ -736,19 +734,6 @@ update_moved_uris_callback (GObject      *object,
     {
         g_warning ("Error updating moved uris: %s", error->message);
     }
-    else
-    {
-        g_autolist (NautilusFile) updated_files = NULL;
-
-        for (guint i = 0; i < new_uris->len; i++)
-        {
-            gchar *new_uri = g_ptr_array_index (new_uris, i);
-
-            updated_files = g_list_prepend (updated_files, nautilus_file_get_by_uri (new_uri));
-        }
-
-        g_signal_emit_by_name (tag_manager, "starred-changed", updated_files);
-    }
 }
 
 /**
@@ -815,7 +800,7 @@ nautilus_tag_manager_update_moved_uris (NautilusTagManager *self,
         return;
     }
 
-    DEBUG ("Updating moved URI for %i starred files", new_uris->len);
+    g_debug ("Updating moved URI for %i starred files", new_uris->len);
 
     query = g_string_new ("DELETE DATA {");
 
@@ -841,15 +826,11 @@ nautilus_tag_manager_update_moved_uris (NautilusTagManager *self,
 
     g_string_append (query, "}");
 
-    /* Forward the new_uris list to later pass in the ::files-changed signal.
-     * There is no need to pass the old_uris because the file model is updated
-     * independently; we need only inform the view where to display stars now.
-     */
     tracker_sparql_connection_update_async (self->db,
                                             query->str,
                                             self->cancellable,
                                             update_moved_uris_callback,
-                                            g_steal_pointer (&new_uris));
+                                            NULL);
 }
 
 static void
@@ -865,7 +846,7 @@ process_tracker2_data_cb (GObject      *source_object,
 
     if (!error)
     {
-        DEBUG ("Data migration was successful. Creating stamp %s", path);
+        g_debug ("Data migration was successful. Creating stamp %s", path);
 
         g_file_set_contents (path, "", -1, &error);
         if (error)
@@ -910,12 +891,12 @@ process_tracker2_data (NautilusTagManager *self,
 
         if (file)
         {
-            DEBUG ("Tracker 2 migration: starring %s", *group);
+            g_debug ("Tracker 2 migration: starring %s", *group);
             selection = g_list_prepend (selection, file);
         }
         else
         {
-            DEBUG ("Tracker 2 migration: couldn't get NautilusFile for %s", *group);
+            g_debug ("Tracker 2 migration: couldn't get NautilusFile for %s", *group);
         }
     }
 
@@ -955,8 +936,8 @@ child_watch_cb (GPid     pid,
                 gint     status,
                 gpointer user_data)
 {
-    DEBUG ("Child %" G_PID_FORMAT " exited %s", pid,
-           g_spawn_check_wait_status (status, NULL) ? "normally" : "abnormally");
+    g_debug ("Child %" G_PID_FORMAT " exited %s", pid,
+             g_spawn_check_wait_status (status, NULL) ? "normally" : "abnormally");
     g_spawn_close_pid (pid);
 }
 
@@ -1009,11 +990,11 @@ nautilus_tag_manager_maybe_migrate_tracker2_data (NautilusTagManager *self)
 
     if (g_file_test (path, G_FILE_TEST_EXISTS))
     {
-        DEBUG ("Tracker 2 migration: already completed.");
+        g_debug ("Tracker 2 migration: already completed.");
     }
     else
     {
-        DEBUG ("Tracker 2 migration: starting.");
+        g_debug ("Tracker 2 migration: starting.");
         export_tracker2_data (self);
     }
 }
