@@ -19,6 +19,7 @@
  *
  *  Author: Andy Hertzfeld <andy@eazel.com>
  */
+#define G_LOG_DOMAIN "nautilus-thumbnails"
 
 #include <config.h>
 #include "nautilus-thumbnails.h"
@@ -37,8 +38,6 @@
 #include <unistd.h>
 #include <signal.h>
 #include <libgnome-desktop/gnome-desktop-thumbnail.h>
-#define DEBUG_FLAG NAUTILUS_DEBUG_THUMBNAILS
-#include "nautilus-debug.h"
 
 #include "nautilus-file-private.h"
 
@@ -86,7 +85,7 @@ static GHashTable *thumbnails_to_make_hash = NULL;
 static GHashTable *currently_thumbnailing_hash = NULL;
 
 /* The number of currently running threads. */
-static gint running_threads = 0;
+static guint running_threads = 0;
 
 /* The maximum number of threads allowed. */
 static guint max_threads = 0;
@@ -212,6 +211,24 @@ nautilus_thumbnail_prioritize (const char *file_uri)
     }
 }
 
+void
+nautilus_thumbnail_deprioritize (const char *file_uri)
+{
+    GList *node;
+
+    if (G_UNLIKELY (thumbnails_to_make_hash == NULL))
+    {
+        return;
+    }
+
+    node = g_hash_table_lookup (thumbnails_to_make_hash, file_uri);
+
+    if (node != NULL)
+    {
+        g_queue_unlink (&thumbnails_to_make, node);
+        g_queue_push_tail_link (&thumbnails_to_make, node);
+    }
+}
 
 /***************************************************************************
  * Thumbnail Thread Functions.
@@ -279,10 +296,9 @@ nautilus_can_thumbnail (NautilusFile *file)
     gboolean res;
     char *uri;
     time_t mtime;
-    char *mime_type;
+    const char *mime_type = nautilus_file_get_mime_type (file);
 
     uri = nautilus_file_get_uri (file);
-    mime_type = nautilus_file_get_mime_type (file);
     mtime = nautilus_file_get_mtime (file);
 
     factory = get_thumbnail_factory ();
@@ -290,7 +306,6 @@ nautilus_can_thumbnail (NautilusFile *file)
                                                          uri,
                                                          mime_type,
                                                          mtime);
-    g_free (mime_type);
     g_free (uri);
 
     return res;
@@ -308,7 +323,7 @@ nautilus_create_thumbnail (NautilusFile *file)
 
     info = g_new0 (NautilusThumbnailInfo, 1);
     info->image_uri = nautilus_file_get_uri (file);
-    info->mime_type = nautilus_file_get_mime_type (file);
+    info->mime_type = g_strdup (nautilus_file_get_mime_type (file));
     info->cancellable = g_cancellable_new ();
 
     /* Hopefully the NautilusFile will already have the image file mtime,
@@ -343,8 +358,8 @@ nautilus_create_thumbnail (NautilusFile *file)
     if (existing == NULL && existing_info == NULL)
     {
         /* Add the thumbnail to the list. */
-        DEBUG ("(Main Thread) Adding thumbnail: %s\n",
-               info->image_uri);
+        g_debug ("(Main Thread) Adding thumbnail: %s",
+                 info->image_uri);
         g_queue_push_tail (&thumbnails_to_make, info);
         node = g_queue_peek_tail_link (&thumbnails_to_make);
         g_hash_table_insert (thumbnails_to_make_hash,
@@ -361,8 +376,8 @@ nautilus_create_thumbnail (NautilusFile *file)
     }
     else
     {
-        DEBUG ("(Main Thread) Updating non-current mtime: %s\n",
-               info->image_uri);
+        g_debug ("(Main Thread) Updating non-current mtime: %s",
+                 info->image_uri);
 
         /* The file in the queue might need a new original mtime */
         if (existing_info == NULL)
@@ -403,7 +418,7 @@ thumbnail_finalize (NautilusThumbnailInfo *info)
 
     if (g_queue_is_empty (&thumbnails_to_make))
     {
-        DEBUG ("(Thumbnail Async Thread) Exiting\n");
+        g_debug ("(Thumbnail Async Thread) Exiting");
     }
     else if (thumbnail_thread_starter_id == 0)
     {
@@ -425,8 +440,8 @@ thumbnail_failed_cb (GObject      *source_object,
                                                                     &error);
     if (error != NULL)
     {
-        DEBUG ("(Thumbnail Async Thread) Could not create a failed thumbnail: %s (%s)\n",
-               info->image_uri, error->message);
+        g_debug ("(Thumbnail Async Thread) Could not create a failed thumbnail: %s (%s)",
+                 info->image_uri, error->message);
     }
 
     thumbnail_finalize (info);
@@ -446,8 +461,8 @@ thumbnail_saved_cb (GObject      *source_object,
                                                            &error);
     if (error != NULL)
     {
-        DEBUG ("(Thumbnail Async Thread) Saving thumbnail failed: %s (%s)\n",
-               info->image_uri, error->message);
+        g_debug ("(Thumbnail Async Thread) Saving thumbnail failed: %s (%s)",
+                 info->image_uri, error->message);
     }
 
     thumbnail_finalize (info);
@@ -470,8 +485,8 @@ thumbnail_generated_cb (GObject      *source_object,
 
     if (g_cancellable_is_cancelled (info->cancellable))
     {
-        DEBUG ("(Thumbnail Async Thread) Cancelled thumbnail: %s\n",
-               info->image_uri);
+        g_debug ("(Thumbnail Async Thread) Cancelled thumbnail: %s",
+                 info->image_uri);
 
         thumbnail_finalize (info);
         return;
@@ -484,8 +499,8 @@ thumbnail_generated_cb (GObject      *source_object,
         g_autofree gchar *mtime = g_strdup_printf ("%" G_GINT64_FORMAT,
                                                    (gint64) info->updated_file_mtime);
 
-        DEBUG ("(Thumbnail Async Thread) Saving thumbnail: %s\n",
-               info->image_uri);
+        g_debug ("(Thumbnail Async Thread) Saving thumbnail: %s",
+                 info->image_uri);
 
         /* This is needed since the attribute is not set on the pixbuf,
          *  only the written thumbnail file.
@@ -503,8 +518,8 @@ thumbnail_generated_cb (GObject      *source_object,
     }
     else
     {
-        DEBUG ("(Thumbnail Async Thread) Thumbnail failed: %s (%s)\n",
-               info->image_uri, error->message);
+        g_debug ("(Thumbnail Async Thread) Thumbnail failed: %s (%s)",
+                 info->image_uri, error->message);
 
         gnome_desktop_thumbnail_factory_create_failed_thumbnail_async (thumbnail_factory,
                                                                        info->image_uri,
@@ -527,14 +542,14 @@ thumbnail_starter_cb (gpointer data)
 {
     GnomeDesktopThumbnailFactory *thumbnail_factory;
     NautilusThumbnailInfo *info = NULL;
-    gint ignored_thumbnails = 0;
+    guint ignored_thumbnails = 0;
     time_t current_orig_mtime = 0;
     time_t current_time;
     guint backoff_time;
     guint backoff_time_min = THUMBNAIL_CREATION_DELAY_SECS + 1;
     GList *node;
 
-    DEBUG ("(Main Thread) Creating thumbnails thread\n");
+    g_debug ("(Main Thread) Creating thumbnails thread");
 
     thumbnail_factory = get_thumbnail_factory ();
     thumbnail_thread_starter_id = 0;
@@ -559,8 +574,8 @@ thumbnail_starter_cb (gpointer data)
         if (current_time < current_orig_mtime + THUMBNAIL_CREATION_DELAY_SECS &&
             current_time >= current_orig_mtime)
         {
-            DEBUG ("(Thumbnail Thread) Skipping: %s\n",
-                   info->image_uri);
+            g_debug ("(Thumbnail Thread) Skipping: %s",
+                     info->image_uri);
 
             /* Only retain the smallest backoff time */
             backoff_time = THUMBNAIL_CREATION_DELAY_SECS - (current_time - current_orig_mtime);
@@ -574,8 +589,8 @@ thumbnail_starter_cb (gpointer data)
         }
 
         /* Create the thumbnail. */
-        DEBUG ("(Thumbnail Thread) Creating thumbnail: %s\n",
-               info->image_uri);
+        g_debug ("(Thumbnail Thread) Creating thumbnail: %s",
+                 info->image_uri);
 
         running_threads += 1;
         g_hash_table_insert (currently_thumbnailing_hash, info->image_uri, info);
