@@ -24,6 +24,7 @@
 #include <gdk/gdk.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
+#include <adwaita.h>
 
 #include "nautilus-types.h"
 
@@ -35,15 +36,14 @@ typedef enum {
 } NautilusLocationChangeType;
 
 #define NAUTILUS_TYPE_WINDOW_SLOT (nautilus_window_slot_get_type ())
-G_DECLARE_FINAL_TYPE (NautilusWindowSlot, nautilus_window_slot, NAUTILUS, WINDOW_SLOT, GtkBox)
+G_DECLARE_FINAL_TYPE (NautilusWindowSlot, nautilus_window_slot, NAUTILUS, WINDOW_SLOT, AdwBin)
 
 typedef struct
 {
-    NautilusFile *file;
-    gint view_before_search;
     GList *back_list;
     GList *forward_list;
     NautilusBookmark *current_location_bookmark;
+    NautilusQuery *current_search_query;
 } NautilusNavigationState;
 
 NautilusWindowSlot * nautilus_window_slot_new              (NautilusWindow     *window);
@@ -80,7 +80,7 @@ gboolean nautilus_window_slot_handle_event       	   (NautilusWindowSlot    *slo
 
 void    nautilus_window_slot_queue_reload		   (NautilusWindowSlot *slot);
 
-GIcon*   nautilus_window_slot_get_icon                     (NautilusWindowSlot *slot);
+const gchar*   nautilus_window_slot_get_icon_name                (NautilusWindowSlot *slot);
 
 const gchar*   nautilus_window_slot_get_tooltip                  (NautilusWindowSlot *slot);
 
@@ -96,7 +96,9 @@ void     nautilus_window_slot_set_active                   (NautilusWindowSlot *
                                                             gboolean            active);
 gboolean nautilus_window_slot_get_loading                  (NautilusWindowSlot *slot);
 
-gboolean nautilus_window_slot_get_searching                (NautilusWindowSlot *slot);
+gboolean nautilus_window_slot_get_search_visible           (NautilusWindowSlot *slot);
+
+gboolean nautilus_window_slot_get_search_global            (NautilusWindowSlot *self);
 
 GList* nautilus_window_slot_get_selection                  (NautilusWindowSlot *slot);
 
@@ -117,4 +119,7 @@ void nautilus_window_slot_back_or_forward                  (NautilusWindowSlot *
                                                             gboolean            back,
                                                             guint               distance);
 
+void nautilus_window_slot_go_up                            (NautilusWindowSlot *slot);
+void nautilus_window_slot_go_down                          (NautilusWindowSlot *slot);
+
 void free_navigation_state                                 (gpointer data);
