@@ -89,7 +89,7 @@ AdwTabView * nautilus_window_get_tab_view (NautilusWindow *window);
 
 void     nautilus_window_show_about_dialog    (NautilusWindow *window);
 
-GtkWidget *nautilus_window_get_toolbar (NautilusWindow *window);
+gboolean    nautilus_window_is_menu_visible       (NautilusWindow *toolbar);
 
 /* sync window GUI with current slot. Used when changing slots,
  * and when updating the slot state.
@@ -101,7 +101,8 @@ void nautilus_window_sync_title            (NautilusWindow *window,
 
 void nautilus_window_show_operation_notification (NautilusWindow *window,
                                                   gchar          *main_label,
-                                                  GFile          *folder_to_open);
+                                                  GFile          *folder_to_open,
+                                                  gboolean        was_quick);
 
 void nautilus_window_search (NautilusWindow *window,
                              NautilusQuery  *query);
